# -*- coding: utf-8 -*-

from qgis.PyQt.QtCore import QCoreApplication
from qgis.core import QgsProcessing, QgsFeatureSink, QgsProcessingException, QgsProcessingAlgorithm, QgsProcessingParameterVectorLayer, QgsProcessingParameterVectorDestination, QgsProcessingParameterBoolean, QgsProcessingParameterNumber # itt még hozzá kell adni a szükséges típusokat
from qgis import processing

class centroid_befoglalo_random(QgsProcessingAlgorithm):
	INPUT = "INPUT" # a bemeneti paraméter neve
	ALL_PARTS = "ALL_PARTS" # a bemeneti paraméter neve
	COUNT = "COUNT" # a bemeneti paraméter neve
	OUTPUT = "OUTPUT" # a kimeneti paraméter neve
	CONVEX_HULL = "CONVEX_HULL" # a kimeneti paraméter neve

	def tr(self, forditando_szoveg):
		return QCoreApplication.translate("Processing", forditando_szoveg)

	def createInstance(self):
		return centroid_befoglalo_random() # itt kell másodszor módosítani az eszköz nevét

	def name(self):
		return "centroid_befoglalo_random"

	def displayName(self):
		return self.tr("Centroidok befoglalójában véletlen pontok képzése")

	def group(self):
		return self.tr("Saját eszközcsoport")

	def groupId(self):
		return "sajat_eszkozcsoport"

	def shortHelpString(self):
		return self.tr("A bemeneti poligonos vektorréteg centroidjaira legkisebb konvex sokszöget húz, amelybe megadott számú véletlen pontot generál.")

	def initAlgorithm(self, config = None):
		self.addParameter( # paraméter hozzáadása
			QgsProcessingParameterVectorLayer( # vektorréteg
				name = self.INPUT,
				description = self.tr("Bemeneti poligonos vektorréteg"),
				types = [QgsProcessing.TypeVectorPolygon]
			)
		)
		self.addParameter( # paraméter hozzáadása
			QgsProcessingParameterBoolean( # bemeneti paraméter
				name = self.ALL_PARTS,
				description = self.tr("A többelemű geometriák egyes részeire külön centroidot képezzek?")
			)
		)
		self.addParameter( # paraméter hozzáadása
			QgsProcessingParameterNumber( # bemeneti paraméter
				name = self.COUNT,
				description = self.tr("Véletlen pontok száma"),
				defaultValue = 1000
			)
		)
		self.addParameter( # paraméter hozzáadása
			QgsProcessingParameterVectorDestination( # kimeneti paraméter
				name = self.OUTPUT,
				description = self.tr("Kimeneti réteg: véletlen pontok")
			)
		)
		self.addParameter( # paraméter hozzáadása
			QgsProcessingParameterVectorDestination( # kimeneti paraméter
				name = self.CONVEX_HULL,
				description = self.tr("Kimeneti réteg: konvex sokszög")
			)
		)

	def processAlgorithm(self, parameters, context, feedback):
		feedback.pushInfo("Az eszköz futása megkezdődött.")
		if feedback.isCanceled():
			return {}
		centroidkepzes_eredmenye = processing.run(
			algOrName = "native:centroids",
			parameters = {
				"INPUT": parameters[self.INPUT],
				"ALL_PARTS": parameters[self.ALL_PARTS],
				"OUTPUT": "TEMPORARY_OUTPUT"
			}, 
			is_child_algorithm = True,
			context = context,
			feedback = feedback)
		if feedback.isCanceled():
			return {}
		feedback.pushInfo("Centroidképzés véget ért, összevonás folyamatban.")
		osszevonas_eredmenye = processing.run(
			algOrName = "native:dissolve",
			parameters = {
				"INPUT": centroidkepzes_eredmenye["OUTPUT"],
				"OUTPUT": "TEMPORARY_OUTPUT"
			}, 
			is_child_algorithm = True,
			context = context,
			feedback = feedback)
		if feedback.isCanceled():
			return {}
		feedback.pushInfo("Összevonás véget ért, konvex sokszöget rajzolása folyamatban.")
		konvexsokszog_rajzolas_eredmenye = processing.run(
			algOrName = "native:convexhull",
			parameters = {
				"INPUT": osszevonas_eredmenye["OUTPUT"],
				"OUTPUT": parameters[self.CONVEX_HULL]
			}, 
			is_child_algorithm = True,
			context = context,
			feedback = feedback)
		if feedback.isCanceled():
			return {}
		feedback.pushInfo("Konvex sokszöget rajzolása véget ért, véletlen pontok generálása folyamatban.")
		pontkepzes_eredmenye = processing.run(
			algOrName = "qgis:randompointsinsidepolygons",
			parameters = {
				"INPUT": konvexsokszog_rajzolas_eredmenye["OUTPUT"],
				"VALUE": parameters[self.COUNT],
				"OUTPUT": parameters[self.OUTPUT]
			}, 
			is_child_algorithm = True,
			context = context,
			feedback = feedback)
		feedback.pushInfo("Az eszköz futása véget ért.")

		return {self.OUTPUT: pontkepzes_eredmenye["OUTPUT"], self.CONVEX_HULL: konvexsokszog_rajzolas_eredmenye["OUTPUT"]}
