# -*- coding: utf-8 -*-

from qgis.PyQt.QtCore import QCoreApplication
from qgis.core import QgsProcessing, QgsFeatureSink, QgsProcessingException, QgsProcessingAlgorithm, QgsProcessingParameterVectorLayer, QgsProcessingParameterVectorDestination, QgsProcessingParameterNumber, QgsProcessingParameterBoolean
 # itt még hozzá kell adni a szükséges típusokat
from qgis import processing

class pufferre_konvex_sokszog(QgsProcessingAlgorithm):
	INPUT = "INPUT" # a bemeneti paraméter neve
	DISTANCE = "DISTANCE" # a második bemeneti paraméter neve
	DISSOLVE = "DISSOLVE" # a harmadik bemeneti paraméter neve
	OUTPUT = "OUTPUT" # a kimeneti paraméter neve
	BUFFER_OUTPUT = "BUFFER_OUTPUT" # a második kimeneti paraméter neve
	
	def tr(self, forditando_szoveg):
		return QCoreApplication.translate("Processing", forditando_szoveg)

	def createInstance(self):
		return pufferre_konvex_sokszog() # itt kell másodszor módosítani az eszköz nevét

	def name(self):
		return "pufferre_konvex_sokszog"

	def displayName(self):
		return self.tr("Puffer és konvex sokszög képzése")

	def group(self):
		return self.tr("Saját eszközcsoport")

	def groupId(self):
		return "sajat_eszkozcsoport"

	def shortHelpString(self):
		return self.tr("Puffert készít megadott távolsággal, majd arra konvex sokszöget rajzol.")

	def initAlgorithm(self, config = None):
		self.addParameter( # paraméter hozzáadása
			QgsProcessingParameterVectorLayer( # bemeneti paraméter
				name = self.INPUT,
				description = self.tr("Bemeneti réteg"),
				types = [QgsProcessing.TypeVectorPolygon]
			)
		)
		self.addParameter( # paraméter hozzáadása
			QgsProcessingParameterNumber( # második bemeneti paraméter
				name = self.DISTANCE,
				description = self.tr("Távolság")
			)
		)
		self.addParameter( # paraméter hozzáadása
			QgsProcessingParameterBoolean( # harmadik bemeneti paraméter
				name = self.DISSOLVE,
				description = self.tr("Végén uniózza-e a különálló puffereket?"),
				defaultValue = False
			)
		)
		self.addParameter( # paraméter hozzáadása
			QgsProcessingParameterVectorDestination( # kimeneti paraméter
				name = self.OUTPUT,
				description = self.tr("Kimeneti réteg")
			)
		)
		self.addParameter( # paraméter hozzáadása
			QgsProcessingParameterVectorDestination( # második kimeneti paraméter
				name = self.BUFFER_OUTPUT,
				description = self.tr("Köztes pufferréteg")
			)
		)

	def processAlgorithm(self, parameters, context, feedback):
		feedback.pushInfo("Pufferképzés megkezdődött.")
		if feedback.isCanceled():
			return {}
		pufferkepzes_eredmenye = processing.run(
			algOrName = "native:buffer",
			parameters = {
				"INPUT": parameters[self.INPUT],
				"DISTANCE": parameters[self.DISTANCE],
				"DISSOLVE": parameters[self.DISSOLVE],
				"OUTPUT": parameters[self.OUTPUT]
			}, 
			is_child_algorithm = True,
			context = context,
			feedback = feedback)
		if feedback.isCanceled():
			return {}
		feedback.pushInfo("Pufferképzés véget ért, konvex sokszög képzése megkezdődött.")
		konvexsokszog_kepzes_eredmenye = processing.run(
			algOrName = "native:convexhull",
			parameters = {
				"INPUT": pufferkepzes_eredmenye["OUTPUT"],
				"OUTPUT": parameters[self.OUTPUT]
			}, 
			is_child_algorithm = True,
			context = context,
			feedback = feedback)
		if feedback.isCanceled():
			return {}
		feedback.pushInfo("A konvex sokszög képzése véget ért.")

		return {self.OUTPUT: konvexsokszog_kepzes_eredmenye["OUTPUT"], self.BUFFER_OUTPUT: pufferkepzes_eredmenye["OUTPUT"]}
